from async_mojang.api import API

from async_mojang.errors import (
    MojangError,
    LoginFailure,
    BadRequest,
    Forbidden,
    NotFound,
    TooManyRequests,
    ServerError,
    Unauthorized,
    MissingMinecraftLicense,
    MissingMinecraftProfile,
)